import exportPlugin from '../../../lib/exportPlugin'

class VimeoAnalytics {
  constructor(player, metadata) {
    this.player = player
    this.metadata = metadata
    this.mostRecentHeartbeat = 0
    this.isPaused = false
  }

  initialize() {
    const { player } = this

    const events = {
      loaded: this.setCurrentVideo,
      play: this.trackPlay,
      pause: this.trackPause,
      ended: this.trackEnded,
      timeupdate: this.trackHeartbeat,
      seeked: this.trackSeek
    }

    for (event in events) {
      this.registerHandler(event, events[event])
    }
  }

  // This is a helper function used to facilitate easier testing.
  registerHandler(event, handler) {
    this.player.on(event, data => handler.call(this, data))
  }

  setCurrentVideo({ id: videoId }) {
    const { player, metadata } = this
    const currentVideoMetadata = metadata.find(m => m.assetId == videoId)
    if (!currentVideoMetadata) {
      console.warn(
        `Could not locate metadata for Vimeo video with id ${videoId}`
      )
      this.currentVideoMetadata = {}
    } else {
      this.currentVideoMetadata = currentVideoMetadata
    }
    this.videoId = videoId
  }

  async trackPlay(data) {
    const { isPaused, currentVideoMetadata } = this
    const props = await this.getPlayerProps(data)
    if (isPaused) {
      window.analytics.track('Video Playback Resumed', props)
      window.analytics.track(
        'Video Content Started',
        Object.assign(currentVideoMetadata, props)
      )
      this.isPaused = false
    } else {
      window.analytics.track('Video Playback Started', props)
      window.analytics.track(
        'Video Content Started',
        Object.assign(currentVideoMetadata, props)
      )
    }
  }

  async trackEnded(data) {
    const props = await this.getPlayerProps(data)
    window.analytics.track('Video Playback Completed', props)
    window.analytics.track(
      'Video Content Completed',
      Object.assign(this.currentVideoMetadata, props)
    )
  }

  async trackHeartbeat(data) {
    // Vimeo provides time updates every 250ms while Segment documents sending heartbeats every 10s.
    // Therefore, we need to do some math to ensure we are not sending 4 heartbeat events when we reach
    // 10 second intervals.
    const mostRecentHeartbeat = this.mostRecentHeartbeat
    const currentPosition = Math.floor(data.seconds)
    if (
      currentPosition !== mostRecentHeartbeat &&
      currentPosition - mostRecentHeartbeat >= 10
    ) {
      const props = await this.getPlayerProps(data)
      window.analytics.track(
        'Video Content Playing',
        Object.assign(props, this.currentVideoMetadata)
      )
      this.mostRecentHeartbeat = currentPosition
    }
    this.mostRecentPosition = currentPosition
  }

  async trackSeek(data) {
    const props = await this.getPlayerProps(data)
    window.analytics.track(
      'Video Playback Seek Started',
      Object.assign({}, props, {
        position: this.mostRecentPosition,
        seekPosition: Math.round(data.seconds)
      })
    )
    window.analytics.track('Video Playback Seek Completed', props)
  }

  async trackPause(data) {
    this.isPaused = true
    const props = await this.getPlayerProps(data)
    window.analytics.track('Video Playback Paused', props)
  }

  async getPlayerProps(data) {
    const { player, currentVideoMetadata, videoId } = this
    const volume = await player.getVolume()

    return {
      videoId,
      sound: Math.round(volume * 100),
      totalLength: Math.round(data.duration),
      sessionId: currentVideoMetadata.sessionId,
      position: Math.round(data.seconds),
      videoPlayer: 'vimeo'
    }
  }
}

export default exportPlugin(VimeoAnalytics)
