import VimeoAnalytics from '../src'
import assert from 'assert'
const sandbox = sinon.createSandbox()

describe('VimeoAnalytics', () => {
  let player
  let vimeoAnalytics

  beforeEach(async () => {
    window.analytics = {
      track: sandbox.spy()
    }

    const html =
      '<iframe id="video-player" src="https://player.vimeo.com/video/76979871" width="640" height="360" frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe>'
    document.body.insertAdjacentHTML('afterbegin', html)
    const iframe = document.querySelector('iframe')
    player = new Vimeo.Player(iframe, { autoplay: true, muted: true })
    vimeoAnalytics = new VimeoAnalytics(player, [
      { assetId: 76979871, title: 'Foobar', sessionId: '1' },
      { assetId: 56282283, title: 'Public Test Video', sessionId: '1' }
    ])
    vimeoAnalytics.initialize()
    await player.loadVideo(76979871)
  })

  afterEach(async () => {
    await player.unload()
    clearEnv('video-player')
    sandbox.reset()
  })

  describe('registerHandler', () => {
    it('should call player.on for any event with the given handler', () => {
      sandbox.spy(player, 'on')
      vimeoAnalytics.registerHandler('play', () => {})
      player.on.should.have.been.calledOnce
      player.on.should.have.been.calledWithMatch(
        sinon.match('play'),
        sinon.match.func
      )
    })
  })

  describe('initialize()', () => {
    it('should initiate all supported event handlers', () => {
      sandbox.spy(vimeoAnalytics, 'registerHandler')

      const events = {
        loaded: vimeoAnalytics.setCurrentVideo,
        play: vimeoAnalytics.trackPlay,
        pause: vimeoAnalytics.trackPause,
        ended: vimeoAnalytics.trackEnded,
        timeupdate: vimeoAnalytics.trackHeartbeat,
        seeked: vimeoAnalytics.trackSeek
      }

      vimeoAnalytics.initialize()

      for (event in events) {
        vimeoAnalytics.registerHandler.should.have.been.calledWith(
          event,
          events[event]
        )
      }
    })
  })

  describe('setCurrentVideo()', () => {
    beforeEach(() => {
      sandbox.spy(vimeoAnalytics, 'setCurrentVideo')
    })

    it('should store the current video id when the video changes', async () => {
      const id = 56282283
      vimeoAnalytics.initialize()
      await player.loadVideo(id)
      vimeoAnalytics.setCurrentVideo.should.have.been.calledOnce
      vimeoAnalytics.setCurrentVideo.should.have.been.calledWith({ id })
      assert.equal(vimeoAnalytics.videoId, 56282283)
    })

    it('should store the current video metadata when the video changes', () => {
      const metadata = vimeoAnalytics.metadata[1]
      vimeoAnalytics.setCurrentVideo({ id: 56282283 })
      assert.deepEqual(vimeoAnalytics.currentVideoMetadata, metadata)
    })

    it('should log when it cannot find metadata for a given video id and store an empty object', () => {
      sandbox.spy(console, 'warn')
      vimeoAnalytics.setCurrentVideo({ id: 'fake-video-id' })
      assert.deepEqual(vimeoAnalytics.currentVideoMetadata, {})
      console.warn.should.have.been.called
    })
  })

  describe('getPlayerProps()', () => {
    const fakeData = {
      duration: 61.857,
      percent: 0.049,
      seconds: 3.034
    }

    beforeEach(async () => {
      sandbox.spy(vimeoAnalytics, 'getPlayerProps')
    })

    it('should return properly formatted player props', async () => {
      await player.setVolume(0.28)
      const expectedProps = {
        videoId: vimeoAnalytics.videoId,
        sound: 28,
        totalLength: 62,
        sessionId: '1',
        position: 3,
        videoPlayer: 'vimeo'
      }
      const props = await vimeoAnalytics.getPlayerProps(fakeData)
      assert.deepEqual(props, expectedProps)
    })
  })

  describe('Event Handlers', () => {
    describe('trackPlay()', () => {
      const playData = {
        duration: 61.857,
        percent: 0,
        seconds: 0
      }

      const pauseData = {
        duration: 61.857,
        percent: 0.049,
        seconds: 3.034
      }

      beforeEach(() => {
        sandbox.spy(vimeoAnalytics, 'trackPlay')
      })

      it('should track play events as Video Playback Started events', async () => {
        await vimeoAnalytics.trackPlay(playData)
        const props = await vimeoAnalytics.getPlayerProps(playData)
        analytics.track.firstCall.should.have.been.calledWith(
          'Video Playback Started',
          props
        )
      })

      it('should not track Video Playback Started events if playback is paused', async () => {
        vimeoAnalytics.isPaused = true
        vimeoAnalytics.trackPlay(pauseData)
        analytics.track.should.not.have.been.calledWith(
          'Video Playback Started'
        )
      })

      it('should track play events as Video Playback Resumed events if the video was previously paused', async () => {
        vimeoAnalytics.isPaused = true
        vimeoAnalytics.trackPlay(pauseData)
        const props = await vimeoAnalytics.getPlayerProps(pauseData)
        analytics.track.should.have.been.calledWith(
          'Video Playback Resumed',
          props
        )
      })

      it('should track play events as Video Content Started events', async () => {
        await vimeoAnalytics.trackPlay(playData)
        const props = await vimeoAnalytics.getPlayerProps(playData)
        analytics.track.should.have.been.calledWith(
          'Video Content Started',
          Object.assign(vimeoAnalytics.currentVideoMetadata, props)
        )
      })

      it('should track play events as Video Content Started events if the video was previously paused', async () => {
        vimeoAnalytics.isPaused = true
        vimeoAnalytics.trackPlay(pauseData)
        const props = await vimeoAnalytics.getPlayerProps(pauseData)
        analytics.track.should.have.been.calledWith(
          'Video Content Started',
          Object.assign(vimeoAnalytics.currentVideoMetadata, props)
        )
      })
    })

    describe('trackPause()', () => {
      const pauseData = {
        duration: 61.857,
        percent: 0.049,
        seconds: 3.034
      }

      beforeEach(() => {
        sandbox.spy(vimeoAnalytics, 'trackPause')
      })

      it('should set isPaused to true', async () => {
        await vimeoAnalytics.trackPause(pauseData)
        assert(vimeoAnalytics.isPaused === true)
      })

      it('should track pause events as Video Playback Paused events', async () => {
        await vimeoAnalytics.trackPause(pauseData)
        analytics.track.should.have.been.calledOnce
        const props = await vimeoAnalytics.getPlayerProps(pauseData)
        analytics.track.should.have.been.calledWith(
          'Video Playback Paused',
          props
        )
      })
    })

    describe('trackHeartbeat()', () => {
      const getData = seconds => {
        seconds = seconds || 0
        const duration = 61.857
        return {
          percent: seconds / duration,
          duration,
          seconds
        }
      }

      beforeEach(() => {
        sandbox.spy(vimeoAnalytics, 'trackHeartbeat')
      })

      it('should not track a heartbeat event if it has not been 10 seconds since the last hearbeat', async () => {
        vimeoAnalytics.mostRecentHeartbeat = 10
        await vimeoAnalytics.trackHeartbeat(getData(15.02))
        analytics.track.should.not.have.been.called
      })

      it('should track a single heartbeat event at 10 second intervals', async () => {
        vimeoAnalytics.mostRecentHeartbeat = 10
        await vimeoAnalytics.trackHeartbeat(getData(20.03))
        await vimeoAnalytics.trackHeartbeat(getData(22.07))
        const props = await vimeoAnalytics.getPlayerProps(getData(22.07))
        analytics.track.should.have.been.calledOnce
        analytics.track.should.not.have.been.calledWith(
          'Video Content Playing',
          props
        )
      })

      it('should update the mostRecentHeartbeat flag when a new heartbeat is triggered', async () => {
        vimeoAnalytics.mostRecentHeartbeat = 20
        await vimeoAnalytics.trackHeartbeat(getData(30.12))
        assert.equal(vimeoAnalytics.mostRecentHeartbeat, 30)
      })

      it('should always update the mostRecentPosition property', async () => {
        vimeoAnalytics.mostRecentHeartbeat = 20
        await vimeoAnalytics.trackHeartbeat(getData(21.12))
        assert.equal(vimeoAnalytics.mostRecentPosition, 21)
      })
    })

    describe('trackSeek()', () => {
      const seekData = {
        duration: 61.857,
        percent: 0.485,
        seconds: 30.13
      }

      beforeEach(() => {
        sandbox.spy(vimeoAnalytics, 'trackSeek')
      })

      it('should track seek events as Video Playback Seek Started events', async () => {
        vimeoAnalytics.mostRecentPosition = 21
        await vimeoAnalytics.trackSeek(seekData)
        const props = await vimeoAnalytics.getPlayerProps(seekData)
        const seekStartedSpy = analytics.track.withArgs(
          'Video Playback Seek Started',
          Object.assign({}, props, {
            seekPosition: Math.round(seekData.seconds),
            position: vimeoAnalytics.mostRecentPosition
          })
        )
        seekStartedSpy.should.have.been.calledOnce
      })

      it('should track seek events as Video Playback Seek Completed events', async () => {
        vimeoAnalytics.mostRecentPosition = 21
        await vimeoAnalytics.trackSeek(seekData)
        const props = await vimeoAnalytics.getPlayerProps(seekData)
        const seekCompletedSpy = analytics.track.withArgs(
          'Video Playback Seek Completed',
          props
        )
        seekCompletedSpy.should.have.been.calledOnce
      })
    })

    describe('trackEnded()', () => {
      const endedData = {
        duration: 61.857,
        percent: 1,
        seconds: 61.857
      }

      beforeEach(() => {
        sandbox.spy(vimeoAnalytics, 'trackEnded')
      })

      it('should track end events as Video Playback Completed events', async () => {
        await vimeoAnalytics.trackEnded(endedData)
        const props = await vimeoAnalytics.getPlayerProps(endedData)
        analytics.track.should.have.been.calledWith(
          'Video Playback Completed',
          props
        )
      })

      it('should track end events as Video Content Completed events', async () => {
        await vimeoAnalytics.trackEnded(endedData)
        const props = await vimeoAnalytics.getPlayerProps(endedData)
        const contentCompletedSpy = analytics.track.withArgs(
          'Video Content Completed',
          Object.assign(vimeoAnalytics.currentVideoMetadata, props)
        )
        contentCompletedSpy.should.have.been.calledOnce
      })
    })
  })
})

function clearEnv(id) {
  document.getElementById(id).remove()
}
